From bb7ddede159d820b6f8b49caf0758940903ece3b Mon Sep 17 00:00:00 2001
From: Kevin Hartman <kevin@hart.mn>
Date: Tue, 2 Sep 2014 22:24:03 -0700
Subject: [PATCH 1/3] Created structure for PAD.

---
 src/core/hle/service/hid.cpp |  3 +++
 src/core/hle/service/hid.h   | 25 +++++++++++++++++++++++++
 2 files changed, 28 insertions(+)

diff --git a/src/core/hle/service/hid.cpp b/src/core/hle/service/hid.cpp
index 4e470795fe..6a4895c901 100644
--- a/src/core/hle/service/hid.cpp
+++ b/src/core/hle/service/hid.cpp
@@ -57,6 +57,9 @@ const Interface::FunctionInfo FunctionTable[] = {
     {0x00170000, nullptr,       "GetSoundVolume"},
 };
 
+
+
+
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 // Interface class
 
diff --git a/src/core/hle/service/hid.h b/src/core/hle/service/hid.h
index b17fcfa865..6ddf2f80f7 100644
--- a/src/core/hle/service/hid.h
+++ b/src/core/hle/service/hid.h
@@ -5,6 +5,7 @@
 #pragma once
 
 #include "core/hle/service/service.h"
+#include "common/bit_field.h"
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 // Namespace HID_User
@@ -14,6 +15,30 @@
 
 namespace HID_User {
 
+struct PADState {
+    union {
+        u32 hex;
+
+        BitField<0, 1, u32> A;
+        BitField<1, 1, u32> B;
+        BitField<2, 1, u32> Select;
+        BitField<3, 1, u32> Start;
+        BitField<4, 1, u32> Right;
+        BitField<5, 1, u32> Left;
+        BitField<6, 1, u32> Up;
+        BitField<7, 1, u32> Down;
+        BitField<8, 1, u32> R;
+        BitField<9, 1, u32> L;
+        BitField<10, 1, u32> X;
+        BitField<11, 1, u32> Y;
+
+        BitField<28, 1, u32> CircleRight;
+        BitField<29, 1, u32> CircleLeft;
+        BitField<30, 1, u32> CircleUp;
+        BitField<31, 1, u32> CircleDown;
+    };
+};
+
 class Interface : public Service::Interface {
 public:
 

From 4a94ec934ab1a2216f94e3fcc46f5dde1d6e2f02 Mon Sep 17 00:00:00 2001
From: Kevin Hartman <kevin@hart.mn>
Date: Wed, 3 Sep 2014 18:12:58 -0700
Subject: [PATCH 2/3] Initial HID PAD work, with GLFW only.

---
 src/citra/emu_window/emu_window_glfw.cpp |  39 ++++++-
 src/common/CMakeLists.txt                |   2 +
 src/common/emu_window.h                  |  19 ++++
 src/common/key_map.cpp                   |  21 ++++
 src/common/key_map.h                     |  35 ++++++
 src/core/hle/service/hid.cpp             | 134 +++++++++++++++++++++--
 src/core/hle/service/hid.h               |  85 +++++++++++---
 7 files changed, 309 insertions(+), 26 deletions(-)
 create mode 100644 src/common/key_map.cpp
 create mode 100644 src/common/key_map.h

diff --git a/src/citra/emu_window/emu_window_glfw.cpp b/src/citra/emu_window/emu_window_glfw.cpp
index 02f524e03c..0a861cff09 100644
--- a/src/citra/emu_window/emu_window_glfw.cpp
+++ b/src/citra/emu_window/emu_window_glfw.cpp
@@ -8,8 +8,35 @@
 
 #include "citra/emu_window/emu_window_glfw.h"
 
-static void OnKeyEvent(GLFWwindow* win, int key, int action) {
-    // TODO(bunnei): ImplementMe
+static const KeyMap::DefaultKeyMapping default_key_map[] = {
+    { KeyMap::CitraKey(GLFW_KEY_A), HID_User::PAD_A },
+    { KeyMap::CitraKey(GLFW_KEY_B), HID_User::PAD_B },
+    { KeyMap::CitraKey(GLFW_KEY_BACKSLASH), HID_User::PAD_SELECT },
+    { KeyMap::CitraKey(GLFW_KEY_ENTER), HID_User::PAD_START },
+    { KeyMap::CitraKey(GLFW_KEY_RIGHT), HID_User::PAD_RIGHT },
+    { KeyMap::CitraKey(GLFW_KEY_LEFT), HID_User::PAD_LEFT },
+    { KeyMap::CitraKey(GLFW_KEY_UP), HID_User::PAD_UP },
+    { KeyMap::CitraKey(GLFW_KEY_DOWN), HID_User::PAD_DOWN },
+    { KeyMap::CitraKey(GLFW_KEY_R), HID_User::PAD_R },
+    { KeyMap::CitraKey(GLFW_KEY_L), HID_User::PAD_L },
+    { KeyMap::CitraKey(GLFW_KEY_X), HID_User::PAD_X },
+    { KeyMap::CitraKey(GLFW_KEY_Y), HID_User::PAD_Y },
+    { KeyMap::CitraKey(GLFW_KEY_H), HID_User::PAD_CIRCLE_RIGHT },
+    { KeyMap::CitraKey(GLFW_KEY_F), HID_User::PAD_CIRCLE_LEFT },
+    { KeyMap::CitraKey(GLFW_KEY_T), HID_User::PAD_CIRCLE_UP },
+    { KeyMap::CitraKey(GLFW_KEY_G), HID_User::PAD_CIRCLE_DOWN },
+};
+
+
+static void OnKeyEvent(GLFWwindow* win, int key, int scancode, int action, int mods) {
+    if (action == GLFW_PRESS) {
+        EmuWindow::KeyPressed(KeyMap::CitraKey(key));
+    }
+
+    if (action == GLFW_RELEASE) {
+        EmuWindow::KeyReleased(KeyMap::CitraKey(key));
+    }
+    HID_User::PADUpdateComplete();
 }
 
 static void OnWindowSizeEvent(GLFWwindow* win, int width, int height) {
@@ -20,6 +47,12 @@ static void OnWindowSizeEvent(GLFWwindow* win, int width, int height) {
 
 /// EmuWindow_GLFW constructor
 EmuWindow_GLFW::EmuWindow_GLFW() {
+
+    // Set default key mappings
+    for (int i = 0; i < ARRAY_SIZE(default_key_map); i++) {
+        KeyMap::SetKeyMapping(default_key_map[i].key, default_key_map[i].state);
+    }
+
     // Initialize the window
     if(glfwInit() != GL_TRUE) {
         printf("Failed to initialize GLFW! Exiting...");
@@ -45,7 +78,7 @@ EmuWindow_GLFW::EmuWindow_GLFW() {
     
     // Setup callbacks
     glfwSetWindowUserPointer(m_render_window, this);
-    //glfwSetKeyCallback(m_render_window, OnKeyEvent);
+    glfwSetKeyCallback(m_render_window, OnKeyEvent);
     //glfwSetWindowSizeCallback(m_render_window, OnWindowSizeEvent);
 
     DoneCurrent();
diff --git a/src/common/CMakeLists.txt b/src/common/CMakeLists.txt
index 3a82f5b807..4ae34bea9e 100644
--- a/src/common/CMakeLists.txt
+++ b/src/common/CMakeLists.txt
@@ -8,6 +8,7 @@ set(SRCS
             file_search.cpp
             file_util.cpp
             hash.cpp
+            key_map.cpp
             log_manager.cpp
             math_util.cpp
             mem_arena.cpp
@@ -39,6 +40,7 @@ set(HEADERS
             file_search.h
             file_util.h
             hash.h
+            key_map.h
             linear_disk_cache.h
             log.h
             log_manager.h
diff --git a/src/common/emu_window.h b/src/common/emu_window.h
index 5e2c33d7ab..90fbd9335a 100644
--- a/src/common/emu_window.h
+++ b/src/common/emu_window.h
@@ -7,6 +7,9 @@
 #include "common/common.h"
 #include "common/scm_rev.h"
 
+#include "common/key_map.h"
+#include "core/hle/service/hid.h"
+
 // Abstraction class used to provide an interface between emulation code and the frontend (e.g. SDL, 
 //  QGLWidget, GLFW, etc...)
 class EmuWindow
@@ -32,6 +35,22 @@ public:
     /// Releases (dunno if this is the "right" word) the GLFW context from the caller thread
     virtual void DoneCurrent() = 0;
 
+    static void KeyPressed(KeyMap::CitraKey key) {
+        HID_User::PADState mapped_key = KeyMap::Get3DSKey(key);
+
+        if (mapped_key.hex != HID_User::PAD_NONE.hex) {
+            HID_User::PADButtonPress(mapped_key);
+        }
+    }
+
+    static void KeyReleased(KeyMap::CitraKey key) {
+        HID_User::PADState mapped_key = KeyMap::Get3DSKey(key);
+
+        if (mapped_key.hex != HID_User::PAD_NONE.hex) {
+            HID_User::PADButtonRelease(mapped_key);
+        }
+    }
+
     Config GetConfig() const { 
         return m_config;
     }
diff --git a/src/common/key_map.cpp b/src/common/key_map.cpp
new file mode 100644
index 0000000000..5941a105b8
--- /dev/null
+++ b/src/common/key_map.cpp
@@ -0,0 +1,21 @@
+// Copyright 2013 Dolphin Emulator Project
+// Licensed under GPLv2
+// Refer to the license.txt file included.
+
+#include "key_map.h"
+#include <map>
+
+
+namespace KeyMap {
+
+std::map<CitraKey, HID_User::PADState> g_key_map;
+
+void SetKeyMapping(CitraKey key, HID_User::PADState padState) {
+    g_key_map[key].hex = padState.hex;
+}
+
+HID_User::PADState Get3DSKey(CitraKey key) {
+    return g_key_map[key];
+}
+
+}
diff --git a/src/common/key_map.h b/src/common/key_map.h
new file mode 100644
index 0000000000..7e94df6181
--- /dev/null
+++ b/src/common/key_map.h
@@ -0,0 +1,35 @@
+// Copyright 2013 Dolphin Emulator Project
+// Licensed under GPLv2
+// Refer to the license.txt file included.
+
+#pragma once
+
+#include "core/hle/service/hid.h"
+
+namespace KeyMap {
+
+class CitraKey {
+public:
+    CitraKey() : keyCode(0) {}
+    CitraKey(int code) : keyCode(code) {}
+
+    int keyCode;
+
+    bool operator < (const CitraKey &other) const {
+        return keyCode < other.keyCode;
+    }
+
+    bool operator == (const CitraKey &other) const {
+        return keyCode == other.keyCode;
+    }
+};
+
+struct DefaultKeyMapping {
+    KeyMap::CitraKey key;
+    HID_User::PADState state;
+};
+
+void SetKeyMapping(CitraKey key, HID_User::PADState padState);
+HID_User::PADState Get3DSKey(CitraKey key);
+
+}
diff --git a/src/core/hle/service/hid.cpp b/src/core/hle/service/hid.cpp
index 6a4895c901..1ef39fced8 100644
--- a/src/core/hle/service/hid.cpp
+++ b/src/core/hle/service/hid.cpp
@@ -16,6 +16,121 @@ namespace HID_User {
 
 Handle g_shared_mem = 0; ///< Handle to shared memory region designated to HID_User service
 
+// Event handles
+Handle g_event_pad_or_touch_1 = 0;
+Handle g_event_pad_or_touch_2 = 0;
+Handle g_event_accelerometer = 0;
+Handle g_event_gyroscope = 0;
+Handle g_event_debug_pad = 0;
+
+// Next PAD state update information
+PADState g_next_state = {{0}};
+u32 g_next_index = 0;
+s16 g_next_circle_x = 0;
+s16 g_next_circle_y = 0;
+
+/** Gets a pointer to the PADData structure inside HID shared memory
+ */
+static inline PADData* GetPADData() {
+    if (0 == g_shared_mem)
+        return nullptr;
+
+    return reinterpret_cast<PADData*>(Kernel::GetSharedMemoryPointer(g_shared_mem, 0));
+}
+
+/** Circle PAD from keys.
+ *
+ *  This is implemented as "pushed all the way to an edge (max) or centered (0)".
+ *
+ *  Indicate the circle pad is pushed completely to the edge in 1 of 8 directions.
+ */
+void UpdateNextCirclePADState() {
+    static const s16 max_value = 0x9C;
+    g_next_circle_x = g_next_state.circle_left ? -max_value : 0x0;
+    g_next_circle_x += g_next_state.circle_right ? max_value : 0x0;
+    g_next_circle_y = g_next_state.circle_down ? -max_value : 0x0;
+    g_next_circle_y += g_next_state.circle_up ? max_value : 0x0;
+}
+
+/** Sets a PAD state (button or button combo) as pressed
+ */
+void PADButtonPress(PADState pad_state) {
+    g_next_state.hex |= pad_state.hex;
+    UpdateNextCirclePADState();
+}
+
+/** Sets a PAD state (button or button combo) as released
+ */
+void PADButtonRelease(PADState pad_state) {
+    g_next_state.hex &= ~pad_state.hex;
+    UpdateNextCirclePADState();
+}
+
+/** Called after all PAD changes to be included in this update have been made,
+ *  including both PAD key changes and analog circle PAD changes.
+ */
+void PADUpdateComplete() {
+    PADData* pad_data = GetPADData();
+
+    // Update PADData struct
+    pad_data->current_state.hex = g_next_state.hex;
+    pad_data->index = g_next_index;
+    g_next_index = (g_next_index + 1) % pad_data->entries.size();
+
+    // Get the previous PAD state
+    u32 last_entry_index = (pad_data->index - 1) % pad_data->entries.size();
+    PADState old_state = pad_data->entries[last_entry_index].current_state;
+
+    // Compute bitmask with 1s for bits different from the old state
+    PADState changed;
+    changed.hex = (g_next_state.hex ^ old_state.hex);
+
+    // Compute what was added
+    PADState additions;
+    additions.hex = changed.hex & g_next_state.hex;
+
+    // Compute what was removed
+    PADState removals;
+    removals.hex = changed.hex & old_state.hex;
+
+    // Get the current PAD entry
+    PADDataEntry* current_pad_entry = &pad_data->entries[pad_data->index];
+
+    // Update entry properties
+    current_pad_entry->current_state.hex = g_next_state.hex;
+    current_pad_entry->delta_additions.hex = additions.hex;
+    current_pad_entry->delta_removals.hex = removals.hex;
+
+    // Set circle PAD
+    current_pad_entry->circle_pad_x = g_next_circle_x;
+    current_pad_entry->circle_pad_y = g_next_circle_y;
+
+    // If we just updated index 0, provide a new timestamp
+    if (pad_data->index == 0) {
+        pad_data->index_reset_ticks_previous = pad_data->index_reset_ticks;
+        pad_data->index_reset_ticks = (s64)Core::g_app_core->GetTicks();
+    }
+
+    // Signal both handles when there's an update to PAD or touch
+    Kernel::SignalEvent(g_event_pad_or_touch_1);
+    Kernel::SignalEvent(g_event_pad_or_touch_2);
+}
+
+
+// TODO(peachum):
+// Add a method for setting analog input from joystick device for the circle PAD.
+//
+// This method should:
+//     * Be called after both PADButton<Press, Release>().
+//     * Be called before PADUpdateComplete()
+//     * Set current PADEntry.circle_pad_<axis> using analog data
+//     * Set PadData.raw_circle_pad_data
+//     * Set PadData.current_state.circle_right = 1 if current PADEntry.circle_pad_x >= 41
+//     * Set PadData.current_state.circle_up = 1 if current PADEntry.circle_pad_y >= 41
+//     * Set PadData.current_state.circle_left = 1 if current PADEntry.circle_pad_x <= -41
+//     * Set PadData.current_state.circle_right = 1 if current PADEntry.circle_pad_y <= -41
+
+
 /**
  * HID_User::GetIPCHandles service function
  *  Inputs:
@@ -35,11 +150,11 @@ void GetIPCHandles(Service::Interface* self) {
 
     cmd_buff[1] = 0; // No error
     cmd_buff[3] = g_shared_mem;
-    cmd_buff[4] = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventA");
-    cmd_buff[5] = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventB");
-    cmd_buff[6] = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventC");
-    cmd_buff[7] = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventGyroscope");
-    cmd_buff[8] = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventD");
+    cmd_buff[4] = g_event_pad_or_touch_1;
+    cmd_buff[5] = g_event_pad_or_touch_2;
+    cmd_buff[6] = g_event_accelerometer;
+    cmd_buff[7] = g_event_gyroscope;
+    cmd_buff[8] = g_event_debug_pad;
 
     DEBUG_LOG(KERNEL, "called");
 }
@@ -58,14 +173,19 @@ const Interface::FunctionInfo FunctionTable[] = {
 };
 
 
-
-
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 // Interface class
 
 Interface::Interface() {
     g_shared_mem = Kernel::CreateSharedMemory("HID_User:SharedMem"); // Create shared memory object
 
+    // Create event handles
+    g_event_pad_or_touch_1 = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventPADOrTouch1");
+    g_event_pad_or_touch_2 = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventPADOrTouch2");
+    g_event_accelerometer = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventAccelerometer");
+    g_event_gyroscope = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventGyroscope");
+    g_event_debug_pad = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventDebugPAD");
+
     Register(FunctionTable, ARRAY_SIZE(FunctionTable));
 }
 
diff --git a/src/core/hle/service/hid.h b/src/core/hle/service/hid.h
index 6ddf2f80f7..f5f76f0fcf 100644
--- a/src/core/hle/service/hid.h
+++ b/src/core/hle/service/hid.h
@@ -15,30 +15,83 @@
 
 namespace HID_User {
 
+/// Structure of a PAD controller state
 struct PADState {
     union {
         u32 hex;
 
-        BitField<0, 1, u32> A;
-        BitField<1, 1, u32> B;
-        BitField<2, 1, u32> Select;
-        BitField<3, 1, u32> Start;
-        BitField<4, 1, u32> Right;
-        BitField<5, 1, u32> Left;
-        BitField<6, 1, u32> Up;
-        BitField<7, 1, u32> Down;
-        BitField<8, 1, u32> R;
-        BitField<9, 1, u32> L;
-        BitField<10, 1, u32> X;
-        BitField<11, 1, u32> Y;
+        BitField<0, 1, u32> a;
+        BitField<1, 1, u32> b;
+        BitField<2, 1, u32> select;
+        BitField<3, 1, u32> start;
+        BitField<4, 1, u32> right;
+        BitField<5, 1, u32> left;
+        BitField<6, 1, u32> up;
+        BitField<7, 1, u32> down;
+        BitField<8, 1, u32> r;
+        BitField<9, 1, u32> l;
+        BitField<10, 1, u32> x;
+        BitField<11, 1, u32> y;
 
-        BitField<28, 1, u32> CircleRight;
-        BitField<29, 1, u32> CircleLeft;
-        BitField<30, 1, u32> CircleUp;
-        BitField<31, 1, u32> CircleDown;
+        BitField<28, 1, u32> circle_right;
+        BitField<29, 1, u32> circle_left;
+        BitField<30, 1, u32> circle_up;
+        BitField<31, 1, u32> circle_down;
     };
 };
 
+/// Structure of a single entry in the PADData's PAD state history array
+struct PADDataEntry {
+    PADState current_state;
+    PADState delta_additions;
+    PADState delta_removals;
+
+    s16 circle_pad_x;
+    s16 circle_pad_y;
+};
+
+/// Structure of all data related to the 3DS Pad
+struct PADData {
+    s64 index_reset_ticks;
+    s64 index_reset_ticks_previous;
+    u32 index; // the index of the last updated PAD state history element
+
+    u32 pad1;
+    u32 pad2;
+
+    PADState current_state; // same as entries[index].current_state
+    u32 raw_circle_pad_data;
+
+    u32 pad3;
+
+    std::array<PADDataEntry, 8> entries; // PAD state history
+};
+
+// Pre-defined PADStates for single button presses
+const PADState PAD_NONE         = {{0}};
+const PADState PAD_A            = {{1u << 0}};
+const PADState PAD_B            = {{1u << 1}};
+const PADState PAD_SELECT       = {{1u << 2}};
+const PADState PAD_START        = {{1u << 3}};
+const PADState PAD_RIGHT        = {{1u << 4}};
+const PADState PAD_LEFT         = {{1u << 5}};
+const PADState PAD_UP           = {{1u << 6}};
+const PADState PAD_DOWN         = {{1u << 7}};
+const PADState PAD_R            = {{1u << 8}};
+const PADState PAD_L            = {{1u << 9}};
+const PADState PAD_X            = {{1u << 10}};
+const PADState PAD_Y            = {{1u << 11}};
+const PADState PAD_CIRCLE_RIGHT = {{1u << 28}};
+const PADState PAD_CIRCLE_LEFT  = {{1u << 29}};
+const PADState PAD_CIRCLE_UP    = {{1u << 30}};
+const PADState PAD_CIRCLE_DOWN  = {{1u << 31}};
+
+// Methods for updating the HID module's state
+void PADButtonPress(PADState pad_state);
+void PADButtonRelease(PADState pad_state);
+void PADUpdateComplete();
+
+/// HID service interface
 class Interface : public Service::Interface {
 public:
 

diff --git a/src/citra/emu_window/emu_window_glfw.h b/src/citra/emu_window/emu_window_glfw.h
index c1b41203be..29325bb756 100644
--- a/src/citra/emu_window/emu_window_glfw.h
+++ b/src/citra/emu_window/emu_window_glfw.h
@@ -25,8 +25,9 @@ public:
     /// Releases (dunno if this is the "right" word) the GLFW context from the caller thread
     void DoneCurrent();
 
-	GLFWwindow* m_render_window;    ///< Internal GLFW render window
+    static void OnKeyEvent(GLFWwindow* win, int key, int scancode, int action, int mods);
 
 private:
-
+    GLFWwindow* m_render_window; ///< Internal GLFW render window
+    int keyboard_id;             ///< Device id of keyboard for use with KeyMap
 };
diff --git a/src/citra_qt/bootmanager.cpp b/src/citra_qt/bootmanager.cpp
index 573060d306..657e39bead 100644
--- a/src/citra_qt/bootmanager.cpp
+++ b/src/citra_qt/bootmanager.cpp
@@ -109,8 +109,35 @@ EmuThread& GRenderWindow::GetEmuThread()
     return emu_thread;
 }
 
+static const std::pair<int, HID_User::PadState> default_key_map[] = {
+    { Qt::Key_A, HID_User::PAD_A },
+    { Qt::Key_B, HID_User::PAD_B },
+    { Qt::Key_Backslash, HID_User::PAD_SELECT },
+    { Qt::Key_Enter, HID_User::PAD_START },
+    { Qt::Key_Right, HID_User::PAD_RIGHT },
+    { Qt::Key_Left, HID_User::PAD_LEFT },
+    { Qt::Key_Up, HID_User::PAD_UP },
+    { Qt::Key_Down, HID_User::PAD_DOWN },
+    { Qt::Key_R, HID_User::PAD_R },
+    { Qt::Key_L, HID_User::PAD_L },
+    { Qt::Key_X, HID_User::PAD_X },
+    { Qt::Key_Y, HID_User::PAD_Y },
+    { Qt::Key_H, HID_User::PAD_CIRCLE_RIGHT },
+    { Qt::Key_F, HID_User::PAD_CIRCLE_LEFT },
+    { Qt::Key_T, HID_User::PAD_CIRCLE_UP },
+    { Qt::Key_G, HID_User::PAD_CIRCLE_DOWN },
+};
+
 GRenderWindow::GRenderWindow(QWidget* parent) : QWidget(parent), emu_thread(this)
 {
+    // Register a new ID for the default keyboard
+    keyboard_id = KeyMap::NewDeviceId();
+
+    // Set default key mappings for keyboard
+    for (auto mapping : default_key_map) {
+        KeyMap::SetKeyMapping({mapping.first, keyboard_id}, mapping.second);
+    }
+
     // TODO: One of these flags might be interesting: WA_OpaquePaintEvent, WA_NoBackground, WA_DontShowOnScreen, WA_DeleteOnClose
     QGLFormat fmt;
     fmt.setProfile(QGLFormat::CoreProfile);
@@ -209,27 +236,13 @@ QByteArray GRenderWindow::saveGeometry()
 
 void GRenderWindow::keyPressEvent(QKeyEvent* event)
 {
-    /*
-    bool key_processed = false;
-    for (unsigned int channel = 0; channel < 4 && controller_interface(); ++channel)
-        if (controller_interface()->SetControllerStatus(channel, event->key(), input_common::GCController::PRESSED))
-            key_processed = true;
-
-    if (!key_processed)
-        QWidget::keyPressEvent(event);
-    */
+    EmuWindow::KeyPressed({event->key(), keyboard_id});
+    HID_User::PadUpdateComplete();
 }
 
 void GRenderWindow::keyReleaseEvent(QKeyEvent* event)
 {
-    /*
-    bool key_processed = false;
-    for (unsigned int channel = 0; channel < 4 && controller_interface(); ++channel)
-        if (controller_interface()->SetControllerStatus(channel, event->key(), input_common::GCController::RELEASED))
-            key_processed = true;
-
-    if (!key_processed)
-        QWidget::keyPressEvent(event);
-    */
+    EmuWindow::KeyReleased({event->key(), keyboard_id});
+    HID_User::PadUpdateComplete();
 }
 
diff --git a/src/citra_qt/bootmanager.hxx b/src/citra_qt/bootmanager.hxx
index 51cb781e91..eedf194711 100644
--- a/src/citra_qt/bootmanager.hxx
+++ b/src/citra_qt/bootmanager.hxx
@@ -116,4 +116,6 @@ private:
     EmuThread emu_thread;
 
     QByteArray geometry;
+
+    int keyboard_id;
 };
diff --git a/src/common/CMakeLists.txt b/src/common/CMakeLists.txt
index 4ae34bea9e..9d5a90762e 100644
--- a/src/common/CMakeLists.txt
+++ b/src/common/CMakeLists.txt
@@ -4,6 +4,7 @@ configure_file("${CMAKE_CURRENT_SOURCE_DIR}/scm_rev.cpp.in" "${CMAKE_CURRENT_SOU
 set(SRCS
             break_points.cpp
             console_listener.cpp
+            emu_window.cpp
             extended_trace.cpp
             file_search.cpp
             file_util.cpp
diff --git a/src/common/emu_window.cpp b/src/common/emu_window.cpp
new file mode 100644
index 0000000000..7a2c50ac83
--- /dev/null
+++ b/src/common/emu_window.cpp
@@ -0,0 +1,17 @@
+// Copyright 2014 Citra Emulator Project
+// Licensed under GPLv2
+// Refer to the license.txt file included.
+
+#include "emu_window.h"
+
+void EmuWindow::KeyPressed(KeyMap::HostDeviceKey key) {
+    HID_User::PadState mapped_key = KeyMap::GetPadKey(key);
+
+    HID_User::PadButtonPress(mapped_key);
+}
+
+void EmuWindow::KeyReleased(KeyMap::HostDeviceKey key) {
+    HID_User::PadState mapped_key = KeyMap::GetPadKey(key);
+
+    HID_User::PadButtonRelease(mapped_key);
+}
diff --git a/src/common/emu_window.h b/src/common/emu_window.h
index 90fbd9335a..23f178fdf8 100644
--- a/src/common/emu_window.h
+++ b/src/common/emu_window.h
@@ -8,7 +8,6 @@
 #include "common/scm_rev.h"
 
 #include "common/key_map.h"
-#include "core/hle/service/hid.h"
 
 // Abstraction class used to provide an interface between emulation code and the frontend (e.g. SDL, 
 //  QGLWidget, GLFW, etc...)
@@ -35,21 +34,11 @@ public:
     /// Releases (dunno if this is the "right" word) the GLFW context from the caller thread
     virtual void DoneCurrent() = 0;
 
-    static void KeyPressed(KeyMap::CitraKey key) {
-        HID_User::PADState mapped_key = KeyMap::Get3DSKey(key);
+    /// Signals a key press action to the HID module
+    static void KeyPressed(KeyMap::HostDeviceKey key);
 
-        if (mapped_key.hex != HID_User::PAD_NONE.hex) {
-            HID_User::PADButtonPress(mapped_key);
-        }
-    }
-
-    static void KeyReleased(KeyMap::CitraKey key) {
-        HID_User::PADState mapped_key = KeyMap::Get3DSKey(key);
-
-        if (mapped_key.hex != HID_User::PAD_NONE.hex) {
-            HID_User::PADButtonRelease(mapped_key);
-        }
-    }
+    /// Signals a key release action to the HID module
+    static void KeyReleased(KeyMap::HostDeviceKey key);
 
     Config GetConfig() const { 
         return m_config;
diff --git a/src/common/key_map.cpp b/src/common/key_map.cpp
index 5941a105b8..309caab986 100644
--- a/src/common/key_map.cpp
+++ b/src/common/key_map.cpp
@@ -1,21 +1,25 @@
-// Copyright 2013 Dolphin Emulator Project
+// Copyright 2014 Citra Emulator Project
 // Licensed under GPLv2
 // Refer to the license.txt file included.
 
 #include "key_map.h"
 #include <map>
 
-
 namespace KeyMap {
 
-std::map<CitraKey, HID_User::PADState> g_key_map;
+static std::map<HostDeviceKey, HID_User::PadState> key_map;
+static int next_device_id = 0;
 
-void SetKeyMapping(CitraKey key, HID_User::PADState padState) {
-    g_key_map[key].hex = padState.hex;
+int NewDeviceId() {
+    return next_device_id++;
 }
 
-HID_User::PADState Get3DSKey(CitraKey key) {
-    return g_key_map[key];
+void SetKeyMapping(HostDeviceKey key, HID_User::PadState padState) {
+    key_map[key].hex = padState.hex;
+}
+
+HID_User::PadState GetPadKey(HostDeviceKey key) {
+    return key_map[key];
 }
 
 }
diff --git a/src/common/key_map.h b/src/common/key_map.h
index 7e94df6181..b5acfbab0d 100644
--- a/src/common/key_map.h
+++ b/src/common/key_map.h
@@ -1,4 +1,4 @@
-// Copyright 2013 Dolphin Emulator Project
+// Copyright 2014 Citra Emulator Project
 // Licensed under GPLv2
 // Refer to the license.txt file included.
 
@@ -8,28 +8,38 @@
 
 namespace KeyMap {
 
-class CitraKey {
-public:
-    CitraKey() : keyCode(0) {}
-    CitraKey(int code) : keyCode(code) {}
+/**
+ * Represents a key for a specific host device.
+ */
+struct HostDeviceKey {
+    int key_code;
+    int device_id; ///< Uniquely identifies a host device
 
-    int keyCode;
-
-    bool operator < (const CitraKey &other) const {
-        return keyCode < other.keyCode;
+    bool operator < (const HostDeviceKey &other) const {
+        if (device_id == other.device_id) {
+            return key_code < other.key_code;
+        }
+        return device_id < other.device_id;
     }
 
-    bool operator == (const CitraKey &other) const {
-        return keyCode == other.keyCode;
+    bool operator == (const HostDeviceKey &other) const {
+        return device_id == other.device_id && key_code == other.key_code;
     }
 };
 
-struct DefaultKeyMapping {
-    KeyMap::CitraKey key;
-    HID_User::PADState state;
-};
+/**
+ * Generates a new device id, which uniquely identifies a host device within KeyMap.
+ */
+int NewDeviceId();
 
-void SetKeyMapping(CitraKey key, HID_User::PADState padState);
-HID_User::PADState Get3DSKey(CitraKey key);
+/**
+ * Maps a device-specific key to a PadState.
+ */
+void SetKeyMapping(HostDeviceKey key, HID_User::PadState padState);
+
+/**
+ * Gets the PadState that's mapped to the provided device-specific key.
+ */
+HID_User::PadState GetPadKey(HostDeviceKey key);
 
 }
diff --git a/src/core/hle/service/hid.cpp b/src/core/hle/service/hid.cpp
index 1ef39fced8..b6ec1b8ff2 100644
--- a/src/core/hle/service/hid.cpp
+++ b/src/core/hle/service/hid.cpp
@@ -14,96 +14,102 @@
 
 namespace HID_User {
 
-Handle g_shared_mem = 0; ///< Handle to shared memory region designated to HID_User service
+// Handle to shared memory region designated to HID_User service
+static Handle shared_mem = 0;
 
 // Event handles
-Handle g_event_pad_or_touch_1 = 0;
-Handle g_event_pad_or_touch_2 = 0;
-Handle g_event_accelerometer = 0;
-Handle g_event_gyroscope = 0;
-Handle g_event_debug_pad = 0;
+static Handle event_pad_or_touch_1 = 0;
+static Handle event_pad_or_touch_2 = 0;
+static Handle event_accelerometer = 0;
+static Handle event_gyroscope = 0;
+static Handle event_debug_pad = 0;
 
-// Next PAD state update information
-PADState g_next_state = {{0}};
-u32 g_next_index = 0;
-s16 g_next_circle_x = 0;
-s16 g_next_circle_y = 0;
+// Next Pad state update information
+static PadState next_state = {{0}};
+static u32 next_index = 0;
+static s16 next_circle_x = 0;
+static s16 next_circle_y = 0;
 
-/** Gets a pointer to the PADData structure inside HID shared memory
+/**
+ * Gets a pointer to the PadData structure inside HID shared memory
  */
-static inline PADData* GetPADData() {
-    if (0 == g_shared_mem)
+static inline PadData* GetPadData() {
+    if (0 == shared_mem)
         return nullptr;
 
-    return reinterpret_cast<PADData*>(Kernel::GetSharedMemoryPointer(g_shared_mem, 0));
+    return reinterpret_cast<PadData*>(Kernel::GetSharedMemoryPointer(shared_mem, 0));
 }
 
-/** Circle PAD from keys.
+/**
+ * Circle Pad from keys.
  *
- *  This is implemented as "pushed all the way to an edge (max) or centered (0)".
+ * This is implemented as "pushed all the way to an edge (max) or centered (0)".
  *
- *  Indicate the circle pad is pushed completely to the edge in 1 of 8 directions.
+ * Indicate the circle pad is pushed completely to the edge in 1 of 8 directions.
  */
-void UpdateNextCirclePADState() {
+void UpdateNextCirclePadState() {
     static const s16 max_value = 0x9C;
-    g_next_circle_x = g_next_state.circle_left ? -max_value : 0x0;
-    g_next_circle_x += g_next_state.circle_right ? max_value : 0x0;
-    g_next_circle_y = g_next_state.circle_down ? -max_value : 0x0;
-    g_next_circle_y += g_next_state.circle_up ? max_value : 0x0;
+    next_circle_x = next_state.circle_left ? -max_value : 0x0;
+    next_circle_x += next_state.circle_right ? max_value : 0x0;
+    next_circle_y = next_state.circle_down ? -max_value : 0x0;
+    next_circle_y += next_state.circle_up ? max_value : 0x0;
 }
 
-/** Sets a PAD state (button or button combo) as pressed
+/**
+ * Sets a Pad state (button or button combo) as pressed
  */
-void PADButtonPress(PADState pad_state) {
-    g_next_state.hex |= pad_state.hex;
-    UpdateNextCirclePADState();
+void PadButtonPress(PadState pad_state) {
+    next_state.hex |= pad_state.hex;
+    UpdateNextCirclePadState();
 }
 
-/** Sets a PAD state (button or button combo) as released
+/**
+ * Sets a Pad state (button or button combo) as released
  */
-void PADButtonRelease(PADState pad_state) {
-    g_next_state.hex &= ~pad_state.hex;
-    UpdateNextCirclePADState();
+void PadButtonRelease(PadState pad_state) {
+    next_state.hex &= ~pad_state.hex;
+    UpdateNextCirclePadState();
 }
 
-/** Called after all PAD changes to be included in this update have been made,
- *  including both PAD key changes and analog circle PAD changes.
+/**
+ * Called after all Pad changes to be included in this update have been made,
+ * including both Pad key changes and analog circle Pad changes.
  */
-void PADUpdateComplete() {
-    PADData* pad_data = GetPADData();
+void PadUpdateComplete() {
+    PadData* pad_data = GetPadData();
 
-    // Update PADData struct
-    pad_data->current_state.hex = g_next_state.hex;
-    pad_data->index = g_next_index;
-    g_next_index = (g_next_index + 1) % pad_data->entries.size();
+    // Update PadData struct
+    pad_data->current_state.hex = next_state.hex;
+    pad_data->index = next_index;
+    next_index = (next_index + 1) % pad_data->entries.size();
 
-    // Get the previous PAD state
+    // Get the previous Pad state
     u32 last_entry_index = (pad_data->index - 1) % pad_data->entries.size();
-    PADState old_state = pad_data->entries[last_entry_index].current_state;
+    PadState old_state = pad_data->entries[last_entry_index].current_state;
 
     // Compute bitmask with 1s for bits different from the old state
-    PADState changed;
-    changed.hex = (g_next_state.hex ^ old_state.hex);
+    PadState changed;
+    changed.hex = (next_state.hex ^ old_state.hex);
 
     // Compute what was added
-    PADState additions;
-    additions.hex = changed.hex & g_next_state.hex;
+    PadState additions;
+    additions.hex = changed.hex & next_state.hex;
 
     // Compute what was removed
-    PADState removals;
+    PadState removals;
     removals.hex = changed.hex & old_state.hex;
 
-    // Get the current PAD entry
-    PADDataEntry* current_pad_entry = &pad_data->entries[pad_data->index];
+    // Get the current Pad entry
+    PadDataEntry* current_pad_entry = &pad_data->entries[pad_data->index];
 
     // Update entry properties
-    current_pad_entry->current_state.hex = g_next_state.hex;
+    current_pad_entry->current_state.hex = next_state.hex;
     current_pad_entry->delta_additions.hex = additions.hex;
     current_pad_entry->delta_removals.hex = removals.hex;
 
-    // Set circle PAD
-    current_pad_entry->circle_pad_x = g_next_circle_x;
-    current_pad_entry->circle_pad_y = g_next_circle_y;
+    // Set circle Pad
+    current_pad_entry->circle_pad_x = next_circle_x;
+    current_pad_entry->circle_pad_y = next_circle_y;
 
     // If we just updated index 0, provide a new timestamp
     if (pad_data->index == 0) {
@@ -111,24 +117,24 @@ void PADUpdateComplete() {
         pad_data->index_reset_ticks = (s64)Core::g_app_core->GetTicks();
     }
 
-    // Signal both handles when there's an update to PAD or touch
-    Kernel::SignalEvent(g_event_pad_or_touch_1);
-    Kernel::SignalEvent(g_event_pad_or_touch_2);
+    // Signal both handles when there's an update to Pad or touch
+    Kernel::SignalEvent(event_pad_or_touch_1);
+    Kernel::SignalEvent(event_pad_or_touch_2);
 }
 
 
 // TODO(peachum):
-// Add a method for setting analog input from joystick device for the circle PAD.
+// Add a method for setting analog input from joystick device for the circle Pad.
 //
 // This method should:
-//     * Be called after both PADButton<Press, Release>().
-//     * Be called before PADUpdateComplete()
-//     * Set current PADEntry.circle_pad_<axis> using analog data
+//     * Be called after both PadButton<Press, Release>().
+//     * Be called before PadUpdateComplete()
+//     * Set current PadEntry.circle_pad_<axis> using analog data
 //     * Set PadData.raw_circle_pad_data
-//     * Set PadData.current_state.circle_right = 1 if current PADEntry.circle_pad_x >= 41
-//     * Set PadData.current_state.circle_up = 1 if current PADEntry.circle_pad_y >= 41
-//     * Set PadData.current_state.circle_left = 1 if current PADEntry.circle_pad_x <= -41
-//     * Set PadData.current_state.circle_right = 1 if current PADEntry.circle_pad_y <= -41
+//     * Set PadData.current_state.circle_right = 1 if current PadEntry.circle_pad_x >= 41
+//     * Set PadData.current_state.circle_up = 1 if current PadEntry.circle_pad_y >= 41
+//     * Set PadData.current_state.circle_left = 1 if current PadEntry.circle_pad_x <= -41
+//     * Set PadData.current_state.circle_right = 1 if current PadEntry.circle_pad_y <= -41
 
 
 /**
@@ -149,12 +155,12 @@ void GetIPCHandles(Service::Interface* self) {
     u32* cmd_buff = Service::GetCommandBuffer();
 
     cmd_buff[1] = 0; // No error
-    cmd_buff[3] = g_shared_mem;
-    cmd_buff[4] = g_event_pad_or_touch_1;
-    cmd_buff[5] = g_event_pad_or_touch_2;
-    cmd_buff[6] = g_event_accelerometer;
-    cmd_buff[7] = g_event_gyroscope;
-    cmd_buff[8] = g_event_debug_pad;
+    cmd_buff[3] = shared_mem;
+    cmd_buff[4] = event_pad_or_touch_1;
+    cmd_buff[5] = event_pad_or_touch_2;
+    cmd_buff[6] = event_accelerometer;
+    cmd_buff[7] = event_gyroscope;
+    cmd_buff[8] = event_debug_pad;
 
     DEBUG_LOG(KERNEL, "called");
 }
@@ -177,14 +183,14 @@ const Interface::FunctionInfo FunctionTable[] = {
 // Interface class
 
 Interface::Interface() {
-    g_shared_mem = Kernel::CreateSharedMemory("HID_User:SharedMem"); // Create shared memory object
+    shared_mem = Kernel::CreateSharedMemory("HID_User:SharedMem"); // Create shared memory object
 
     // Create event handles
-    g_event_pad_or_touch_1 = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventPADOrTouch1");
-    g_event_pad_or_touch_2 = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventPADOrTouch2");
-    g_event_accelerometer = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventAccelerometer");
-    g_event_gyroscope = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventGyroscope");
-    g_event_debug_pad = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventDebugPAD");
+    event_pad_or_touch_1 = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventPadOrTouch1");
+    event_pad_or_touch_2 = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventPadOrTouch2");
+    event_accelerometer = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventAccelerometer");
+    event_gyroscope = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventGyroscope");
+    event_debug_pad = Kernel::CreateEvent(RESETTYPE_ONESHOT, "HID_User:EventDebugPad");
 
     Register(FunctionTable, ARRAY_SIZE(FunctionTable));
 }
diff --git a/src/video_core/video_core.h b/src/video_core/video_core.h
index d227b6aa42..5e8129b5a4 100644
--- a/src/video_core/video_core.h
+++ b/src/video_core/video_core.h
@@ -27,6 +27,7 @@ static const int kScreenBottomHeight    = 240;  ///< 3DS bottom screen height
 
 extern RendererBase*   g_renderer;              ///< Renderer plugin
 extern int             g_current_frame;         ///< Current frame
+extern EmuWindow*      g_emu_window;            ///< Emu window
 
 /// Start the video core
 void Start();
From 02fd19b2f60f4db8a683734e4300d7498c861309 Mon Sep 17 00:00:00 2001
From: Kevin Hartman <kevin@hart.mn>
Date: Mon, 8 Sep 2014 21:46:02 -0700
Subject: [PATCH 3/3] Added support for multiple input device types for KeyMap
 and connected Qt.

---
 src/citra/emu_window/emu_window_glfw.cpp |  65 +++++-----
 src/citra/emu_window/emu_window_glfw.h   |   5 +-
 src/citra_qt/bootmanager.cpp             |  49 ++++---
 src/citra_qt/bootmanager.hxx             |   2 +
 src/common/CMakeLists.txt                |   1 +
 src/common/emu_window.cpp                |  17 +++
 src/common/emu_window.h                  |  19 +--
 src/common/key_map.cpp                   |  18 ++-
 src/common/key_map.h                     |  44 ++++---
 src/core/hle/service/hid.cpp             | 156 ++++++++++++-----------
 src/core/hle/service/hid.h               |  80 ++++++------
 src/video_core/video_core.h              |   1 +
 12 files changed, 256 insertions(+), 201 deletions(-)
 create mode 100644 src/common/emu_window.cpp

diff --git a/src/citra/emu_window/emu_window_glfw.cpp b/src/citra/emu_window/emu_window_glfw.cpp
index 0a861cff09..b911e60c5d 100644
--- a/src/citra/emu_window/emu_window_glfw.cpp
+++ b/src/citra/emu_window/emu_window_glfw.cpp
@@ -8,49 +8,53 @@
 
 #include "citra/emu_window/emu_window_glfw.h"
 
-static const KeyMap::DefaultKeyMapping default_key_map[] = {
-    { KeyMap::CitraKey(GLFW_KEY_A), HID_User::PAD_A },
-    { KeyMap::CitraKey(GLFW_KEY_B), HID_User::PAD_B },
-    { KeyMap::CitraKey(GLFW_KEY_BACKSLASH), HID_User::PAD_SELECT },
-    { KeyMap::CitraKey(GLFW_KEY_ENTER), HID_User::PAD_START },
-    { KeyMap::CitraKey(GLFW_KEY_RIGHT), HID_User::PAD_RIGHT },
-    { KeyMap::CitraKey(GLFW_KEY_LEFT), HID_User::PAD_LEFT },
-    { KeyMap::CitraKey(GLFW_KEY_UP), HID_User::PAD_UP },
-    { KeyMap::CitraKey(GLFW_KEY_DOWN), HID_User::PAD_DOWN },
-    { KeyMap::CitraKey(GLFW_KEY_R), HID_User::PAD_R },
-    { KeyMap::CitraKey(GLFW_KEY_L), HID_User::PAD_L },
-    { KeyMap::CitraKey(GLFW_KEY_X), HID_User::PAD_X },
-    { KeyMap::CitraKey(GLFW_KEY_Y), HID_User::PAD_Y },
-    { KeyMap::CitraKey(GLFW_KEY_H), HID_User::PAD_CIRCLE_RIGHT },
-    { KeyMap::CitraKey(GLFW_KEY_F), HID_User::PAD_CIRCLE_LEFT },
-    { KeyMap::CitraKey(GLFW_KEY_T), HID_User::PAD_CIRCLE_UP },
-    { KeyMap::CitraKey(GLFW_KEY_G), HID_User::PAD_CIRCLE_DOWN },
+static const std::pair<int, HID_User::PadState> default_key_map[] = {
+    { GLFW_KEY_A, HID_User::PAD_A },
+    { GLFW_KEY_B, HID_User::PAD_B },
+    { GLFW_KEY_BACKSLASH, HID_User::PAD_SELECT },
+    { GLFW_KEY_ENTER, HID_User::PAD_START },
+    { GLFW_KEY_RIGHT, HID_User::PAD_RIGHT },
+    { GLFW_KEY_LEFT, HID_User::PAD_LEFT },
+    { GLFW_KEY_UP, HID_User::PAD_UP },
+    { GLFW_KEY_DOWN, HID_User::PAD_DOWN },
+    { GLFW_KEY_R, HID_User::PAD_R },
+    { GLFW_KEY_L, HID_User::PAD_L },
+    { GLFW_KEY_X, HID_User::PAD_X },
+    { GLFW_KEY_Y, HID_User::PAD_Y },
+    { GLFW_KEY_H, HID_User::PAD_CIRCLE_RIGHT },
+    { GLFW_KEY_F, HID_User::PAD_CIRCLE_LEFT },
+    { GLFW_KEY_T, HID_User::PAD_CIRCLE_UP },
+    { GLFW_KEY_G, HID_User::PAD_CIRCLE_DOWN },
 };
 
+/// Called by GLFW when a key event occurs
+void EmuWindow_GLFW::OnKeyEvent(GLFWwindow* win, int key, int scancode, int action, int mods) {
+
+    if (!VideoCore::g_emu_window) {
+        return;
+    }
+
+    int keyboard_id = ((EmuWindow_GLFW*)VideoCore::g_emu_window)->keyboard_id;
 
-static void OnKeyEvent(GLFWwindow* win, int key, int scancode, int action, int mods) {
     if (action == GLFW_PRESS) {
-        EmuWindow::KeyPressed(KeyMap::CitraKey(key));
+        EmuWindow::KeyPressed({key, keyboard_id});
     }
 
     if (action == GLFW_RELEASE) {
-        EmuWindow::KeyReleased(KeyMap::CitraKey(key));
+        EmuWindow::KeyReleased({key, keyboard_id});
     }
-    HID_User::PADUpdateComplete();
-}
-
-static void OnWindowSizeEvent(GLFWwindow* win, int width, int height) {
-    EmuWindow_GLFW* emu_window = (EmuWindow_GLFW*)glfwGetWindowUserPointer(win);
-    emu_window->SetClientAreaWidth(width);
-    emu_window->SetClientAreaHeight(height);
+    HID_User::PadUpdateComplete();
 }
 
 /// EmuWindow_GLFW constructor
 EmuWindow_GLFW::EmuWindow_GLFW() {
 
-    // Set default key mappings
-    for (int i = 0; i < ARRAY_SIZE(default_key_map); i++) {
-        KeyMap::SetKeyMapping(default_key_map[i].key, default_key_map[i].state);
+    // Register a new ID for the default keyboard
+    keyboard_id = KeyMap::NewDeviceId();
+
+    // Set default key mappings for keyboard
+    for (auto mapping : default_key_map) {
+        KeyMap::SetKeyMapping({mapping.first, keyboard_id}, mapping.second);
     }
 
     // Initialize the window
@@ -79,7 +83,6 @@ EmuWindow_GLFW::EmuWindow_GLFW() {
     // Setup callbacks
     glfwSetWindowUserPointer(m_render_window, this);
     glfwSetKeyCallback(m_render_window, OnKeyEvent);
-    //glfwSetWindowSizeCallback(m_render_window, OnWindowSizeEvent);
 
     DoneCurrent();
 }
diff --git a/src/core/hle/service/hid.h b/src/core/hle/service/hid.h
index f5f76f0fcf..a077e25cd3 100644
--- a/src/core/hle/service/hid.h
+++ b/src/core/hle/service/hid.h
@@ -10,13 +10,15 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////
 // Namespace HID_User
 
-// This service is used for interfacing to physical user controls... perhaps "Human Interface 
-// Devices"? Uses include game pad controls, accelerometers, gyroscopes, etc.
+// This service is used for interfacing to physical user controls.
+// Uses include game pad controls, touchscreen, accelerometers, gyroscopes, and debug pad.
 
 namespace HID_User {
 
-/// Structure of a PAD controller state
-struct PADState {
+/** 
+ * Structure of a Pad controller state.
+ */
+struct PadState {
     union {
         u32 hex;
 
@@ -40,58 +42,64 @@ struct PADState {
     };
 };
 
-/// Structure of a single entry in the PADData's PAD state history array
-struct PADDataEntry {
-    PADState current_state;
-    PADState delta_additions;
-    PADState delta_removals;
+/**
+ * Structure of a single entry in the PadData's Pad state history array.
+ */
+struct PadDataEntry {
+    PadState current_state;
+    PadState delta_additions;
+    PadState delta_removals;
 
     s16 circle_pad_x;
     s16 circle_pad_y;
 };
 
-/// Structure of all data related to the 3DS Pad
-struct PADData {
+/**
+ * Structure of all data related to the 3DS Pad.
+ */
+struct PadData {
     s64 index_reset_ticks;
     s64 index_reset_ticks_previous;
-    u32 index; // the index of the last updated PAD state history element
+    u32 index; // the index of the last updated Pad state history element
 
     u32 pad1;
     u32 pad2;
 
-    PADState current_state; // same as entries[index].current_state
+    PadState current_state; // same as entries[index].current_state
     u32 raw_circle_pad_data;
 
     u32 pad3;
 
-    std::array<PADDataEntry, 8> entries; // PAD state history
+    std::array<PadDataEntry, 8> entries; // Pad state history
 };
 
-// Pre-defined PADStates for single button presses
-const PADState PAD_NONE         = {{0}};
-const PADState PAD_A            = {{1u << 0}};
-const PADState PAD_B            = {{1u << 1}};
-const PADState PAD_SELECT       = {{1u << 2}};
-const PADState PAD_START        = {{1u << 3}};
-const PADState PAD_RIGHT        = {{1u << 4}};
-const PADState PAD_LEFT         = {{1u << 5}};
-const PADState PAD_UP           = {{1u << 6}};
-const PADState PAD_DOWN         = {{1u << 7}};
-const PADState PAD_R            = {{1u << 8}};
-const PADState PAD_L            = {{1u << 9}};
-const PADState PAD_X            = {{1u << 10}};
-const PADState PAD_Y            = {{1u << 11}};
-const PADState PAD_CIRCLE_RIGHT = {{1u << 28}};
-const PADState PAD_CIRCLE_LEFT  = {{1u << 29}};
-const PADState PAD_CIRCLE_UP    = {{1u << 30}};
-const PADState PAD_CIRCLE_DOWN  = {{1u << 31}};
+// Pre-defined PadStates for single button presses
+const PadState PAD_NONE         = {{0}};
+const PadState PAD_A            = {{1u << 0}};
+const PadState PAD_B            = {{1u << 1}};
+const PadState PAD_SELECT       = {{1u << 2}};
+const PadState PAD_START        = {{1u << 3}};
+const PadState PAD_RIGHT        = {{1u << 4}};
+const PadState PAD_LEFT         = {{1u << 5}};
+const PadState PAD_UP           = {{1u << 6}};
+const PadState PAD_DOWN         = {{1u << 7}};
+const PadState PAD_R            = {{1u << 8}};
+const PadState PAD_L            = {{1u << 9}};
+const PadState PAD_X            = {{1u << 10}};
+const PadState PAD_Y            = {{1u << 11}};
+const PadState PAD_CIRCLE_RIGHT = {{1u << 28}};
+const PadState PAD_CIRCLE_LEFT  = {{1u << 29}};
+const PadState PAD_CIRCLE_UP    = {{1u << 30}};
+const PadState PAD_CIRCLE_DOWN  = {{1u << 31}};
 
 // Methods for updating the HID module's state
-void PADButtonPress(PADState pad_state);
-void PADButtonRelease(PADState pad_state);
-void PADUpdateComplete();
+void PadButtonPress(PadState pad_state);
+void PadButtonRelease(PadState pad_state);
+void PadUpdateComplete();
 
-/// HID service interface
+/**
+ * HID service interface.
+ */
 class Interface : public Service::Interface {
 public:
 